#!/usr/bin/Rscript
###
### Download and extract the required weather data from the DWD archives.
###
### Daniel Vedder, original 27/07/2023, revised 11/2/2025
### 

library(tidyverse)
library(rdwd)

## See here for rdwd docs:
## https://bookdown.org/brry/rdwd/
## https://www.rdocumentation.org/packages/rdwd/versions/1.8.0
## https://www.dwd.de/DE/leistungen/cdc/cdc_ueberblick-klimadaten.html?nn=16102

#rdwd::updateRdwd() # run this now and again to make sure we have the latest file index

region_to_stationid = c(
    "Jena"             = 2444,
    "Eichsfeld"        = 2925,
    "Thüringer Becken" = 896,
    "Hohenlohe"        = 3761,
    "Bodensee"         = 6263,
    "Oberrhein"        = 5275
)

download_data = function(
    region,                               # e.g. "Jena"
    startdate = as.Date("1990-01-01"),    # earliest date to include (if available)
    enddate = as.Date("2024-12-31")       # latest date to include (if available)
) {
    ## DOWNLOAD DATA

    ### observed climate data (these include most of our parameters)

    regions = names(region_to_stationid)
    if (! region %in% regions) {
        cat("Available regions: ", regions, "\n")
        stop(paste("Region ", region), " not found in available regions.\n")
    }
    stationid = region_to_stationid[region]
    observed_url = selectDWD(id = stationid,
                             res = "daily",
                             per = "historical",
                             var = "kl")

    climdata = dataDWD(observed_url, dir = getwd()) %>% as_tibble

    ### derived agrometeorological variables (needed for the potential evapotranspiration)
    ### (see https://bookdown.org/brry/rdwd/use-case-derived-data.html)

    deriv_base = "ftp://opendata.dwd.de/climate_environment/CDC/derived_germany"
    soil_index = indexFTP(folder="soil/daily", base=deriv_base) %>%
        createIndex(base=deriv_base)
    colnames(soil_index)[1:2] = c("var", "res") # inverted column order in this folder

    derived_url = selectDWD(id = stationid[region],
                            res = "daily",
                            per = "historical",
                            var = "soil",
                            base = deriv_base,
                            findex = soil_index)

    soildata = dataDWD(derived_url, base = deriv_base, dir = getwd())

    ## PROCESS DATA

    weather = climdata %>%
        ## drop values outside of the specified date range
	filter(MESS_DATUM >= startdate, MESS_DATUM <= enddate) %>%
        ## select relevant variables and convert place-holder values to NA
        select(MESS_DATUM, FM, RSK, SDK, NM, TMK, TXK, TNK, UPM) %>%
        mutate(date=MESS_DATUM, MESS_DATUM=NULL,
               mean_windspeed=FM, FM=NULL,
               precipitation=RSK, RSK=NULL,
               sunshine_hours=SDK, SDK=NULL,
               mean_cloud_cover=NM, NM=NULL,
               mean_humidity=UPM, UPM=NULL,
               mean_temperature=TMK, TMK=NULL,
               max_temperature=TXK, TXK=NULL,
               min_temperature=TNK, TNK=NULL)

    firstdate = min(weather$date, na.rm = TRUE)
    lastdate = max(weather$date, na.rm = TRUE)
    all_dates = tibble(date = seq.Date(firstdate, lastdate, by = "day"))

    ETo = soildata[names(soildata)[grep("v2", names(soildata))]][[1]] %>%
        as_tibble %>% select(Datum, VPGFAO) %>%
        filter(Datum >= firstdate, Datum <= lastdate) %>%
        mutate(potential_evapotranspiration=VPGFAO, VPGFAO=NULL)

    weather = all_dates %>%
        left_join(weather, by="date") %>%
        left_join(ETo %>% rename(date = Datum), by="date")

    return(weather)
}

main = function(args, script_name = "extract_weather_data.R") {
    print_usage = function(script_name, args) {
        cat(paste("usage:", script_name), "region1 [region2] ...\n")
        cat(paste("      ", script_name), "--all\n")
    }
    if (length(args) == 0) {
        cat("error: not enough arguments\n")
        print_usage(script_name, args)
        return(-1)
    }
    if ("-h" %in% args || "--help" %in% args) {
        print_usage(script_name, args)
        return(0)
    }
    if ("--all" %in% args) {
        regions = names(region_to_stationid)
    } else {
        regions = args
    }
    for (region in regions) {
        weather = download_data(region)
        write.csv(weather, file=paste0("weather_", gsub(" ", "_", region), ".csv"), row.names=FALSE)
    }
    return(0)
}

# Run main function if executed directly
if (sys.nframe() == 0) {
    args = commandArgs(trailingOnly = TRUE)
    cat("\nRegions = ", args, "\n\n")
    status = main(args)
    quit(status = status)
}
